using System.Windows.Forms;
using Cruiser.DrawningObjects;
using Cruiser.MovementStrategy;

namespace Cruiser
{
    public partial class CruiserForm : Form
    {
        private DrawningCruiser? _drawningCruiser;
        
        private AbstractStrategy? _abstractStrategy;
        public DrawningCruiser? SelectedCruiser { get; private set; }
        public CruiserForm()
        {
            InitializeComponent();
            _abstractStrategy = null;
            SelectedCruiser = null;
        }
        private void Draw()
        {
            if (_drawningCruiser == null)
            {
                return;
            }
            Bitmap bmp = new Bitmap(pictureBoxCruiser.Width, pictureBoxCruiser.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _drawningCruiser.DrawTransport(gr);
            pictureBoxCruiser.Image = bmp;
        }
        private void ButtonMove_Click(object sender, EventArgs e)
        {
            if (_drawningCruiser == null)
            {
                return;
            }
            string name = ((Button)sender)?.Name ?? string.Empty;
            switch (name)
            {
                case "buttonUp":
                    _drawningCruiser.MoveTransport(Direction.Up);
                    break;
                case "buttonDown":
                    _drawningCruiser.MoveTransport(Direction.Down);
                    break;
                case "buttonLeft":
                    _drawningCruiser.MoveTransport(Direction.Left);
                    break;
                case "buttonRight":
                    _drawningCruiser.MoveTransport(Direction.Right);
                    break;
            }
            Draw();
        }
        private void ButtonCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogColor = new();
            if (dialogColor.ShowDialog() == DialogResult.OK)
            {
                color = dialogColor.Color;
            }
            _drawningCruiser = new DrawningCruiser(random.Next(100, 300), random.Next(1000, 3000),
            color, pictureBoxCruiser.Width, pictureBoxCruiser.Height);
            _drawningCruiser.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
    private void ButtonStep_Click(object sender, EventArgs e)
        {
            if (_drawningCruiser == null)
            {
                return;
            }
            if (comboBoxCruiser.Enabled)
            {
                _abstractStrategy = comboBoxCruiser.SelectedIndex
                switch
                {
                    0 => new MoveToCenter(),
                    1 => new MoveToBorder(),
                    _ => null,
                };
                if (_abstractStrategy == null)
                {
                    return;
                }
                _abstractStrategy.SetData(new DrawningObjectCruiser(_drawningCruiser), pictureBoxCruiser.Width,
                pictureBoxCruiser.Height);
            }
            if (_abstractStrategy == null)
            {
                return;
            }
            _abstractStrategy.MakeStep();
            Draw();
            if (_abstractStrategy.GetStatus() == Status.Finish)
            {
                comboBoxCruiser.Enabled = true;
                _abstractStrategy = null;
            }
        }
        private void ButtonAdvancedCreate_Click(object sender, EventArgs e)
        {
            Random random = new();
            Color color = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogColor = new();
            if (dialogColor.ShowDialog() == DialogResult.OK)
            {
                color = dialogColor.Color;
            }

            Color dopColor = Color.FromArgb(random.Next(0, 256),
            random.Next(0, 256), random.Next(0, 256));
            ColorDialog dialogDopColor = new();
            if (dialogDopColor.ShowDialog() == DialogResult.OK)
            {
                dopColor = dialogDopColor.Color;
            }

            _drawningCruiser = new DrawningAdvancedCruiser(random.Next(100, 300),
            random.Next(1000, 3000), color, dopColor, Convert.ToBoolean(random.Next(0, 2)),
            Convert.ToBoolean(random.Next(0, 2)),
            pictureBoxCruiser.Width, pictureBoxCruiser.Height);
            _drawningCruiser.SetPosition(random.Next(10, 100), random.Next(10, 100));
            Draw();
        }
        private void ButtonSelectedCruiser_Click(object sender, EventArgs e)
        {
            SelectedCruiser = _drawningCruiser;
            DialogResult = DialogResult.OK;
        }
    }
}