﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Cruiser.Entities;
using Cruiser.MovementStrategy;

namespace Cruiser.DrawningObjects
{
    public class DrawningCruiser
    {
        public EntityCruiser? EntityCruiser { get; protected set; }
        private int _pictureWidth;
        private int _pictureHeight;
        protected int _startPosX;
        protected int _startPosY;
        private readonly int _cruiserWidth = 110;
        private readonly int _cruiserHeight = 60;
        public int GetPosX => _startPosX;
        public int GetPosY => _startPosY;
        public int GetWidth => _cruiserWidth;
        public int GetHeight => _cruiserHeight;
        public IMoveableObject GetMoveableObject => new DrawningObjectCruiser(this);
        public DrawningCruiser(int speed, double weight, Color bodyColor, int width, int height)
        {
            if (width < _cruiserWidth || height < _cruiserHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            EntityCruiser = new EntityCruiser(speed, weight, bodyColor);
        }
        protected DrawningCruiser(int speed, double weight, Color bodyColor, int
width, int height, int cruiserWidth, int cruiserHeight)
        {
            if (width <= _cruiserWidth || height <= _cruiserHeight)
            {
                return;
            }
            _pictureWidth = width;
            _pictureHeight = height;
            _cruiserWidth = cruiserWidth;
            _cruiserHeight = cruiserHeight;
            EntityCruiser = new EntityCruiser(speed, weight, bodyColor);
        }
        public void SetPosition(int x, int y)
        {
            if (x < 0 || x + _cruiserWidth > _pictureWidth)
            {
                x = Math.Max(0, _pictureWidth - _cruiserWidth);
            }
            if (y < 0 || y + _cruiserHeight > _pictureHeight)
            {
                y = Math.Max(0, _pictureHeight - _cruiserHeight);
            }
            _startPosX = x;
            _startPosY = y;
        }
        public void MoveTransport(Direction direction)
        {
            if (!CanMove(direction) || EntityCruiser == null)
            {
                return;
            }
            switch (direction)
            {
                case Direction.Left:
                    _startPosX -= (int)EntityCruiser.Step;
                    break;
                case Direction.Up:
                    _startPosY -= (int)EntityCruiser.Step;
                    break;
                case Direction.Right:
                    _startPosX += (int)EntityCruiser.Step;
                    break;
                case Direction.Down:
                    _startPosY += (int)EntityCruiser.Step;
                    break;
            }
        }
        public bool CanMove(Direction direction)
        {
            if (EntityCruiser == null)
            {
                return false;
            }
            return direction switch
            {
                Direction.Left => _startPosX - EntityCruiser.Step > 0,
                Direction.Up => _startPosY - EntityCruiser.Step > 0,
                Direction.Right => _startPosX + EntityCruiser.Step + _cruiserWidth < _pictureWidth,
                Direction.Down => _startPosY + EntityCruiser.Step + _cruiserHeight < _pictureHeight,
                _ => false,
            };
        }
        public virtual void DrawTransport(Graphics g)
        {
            if (EntityCruiser == null)
            {
                return;
            }
            Pen pen = new Pen(Color.Black);
            Brush brush = new SolidBrush(EntityCruiser.BodyColor);
            g.DrawEllipse(pen, _startPosX + 15, _startPosY + 5, 20, 20);
            g.DrawEllipse(pen, _startPosX + 15, _startPosY + 35, 20, 20);
            g.DrawRectangle(pen, _startPosX + 9, _startPosY + 15, 10, 30);
            g.DrawRectangle(pen, _startPosX + 90, _startPosY + 15, 10,
           30);
            g.DrawRectangle(pen, _startPosX + 20, _startPosY + 4, 70, 52);
            Brush br = new SolidBrush(EntityCruiser.BodyColor);
            g.FillRectangle(br, _startPosX + 10, _startPosY + 15, 10, 30);
            g.FillRectangle(br, _startPosX + 90, _startPosY + 15, 10, 30);
            g.FillRectangle(br, _startPosX + 20, _startPosY + 5, 70, 50);
            Point[] points = new Point[3];//
            points[0] = new Point(_startPosX + 100, _startPosY + 5);
            points[1] = new Point(_startPosX + 100, _startPosY + 55);
            points[2] = new Point(_startPosX + 100 + 50, _startPosY + 50 / 2);
            g.FillPolygon(Brushes.Pink, points);
            g.FillRectangle(Brushes.Black, _startPosX + 5, _startPosY + 15, 10, 10);
            g.FillRectangle(Brushes.Black, _startPosX + 5, _startPosY + 35, 10, 10);
            g.DrawRectangle(Pens.Black, _startPosX + 35,
               _startPosY + 23, 15, 15);
            g.DrawRectangle(Pens.Black, _startPosX + 50,
            _startPosY + 19, 30, 25);
        }
    }
}