﻿using Cruiser.Generics;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Cruiser
{
    public partial class FormCruiserCollection : Form
    {
        private readonly CruiserGenericStorage _storage;
        
        public FormCruiserCollection()
        {
            InitializeComponent();
            _storage = new CruiserGenericStorage(pictureBoxCruiser.Width, pictureBoxCruiser.Height);
        }
        private void ReloadObjects()
        {
            int index = listBoxStorages.SelectedIndex;

            listBoxStorages.Items.Clear();
            for (int i = 0; i < _storage.Keys.Count; i++)
            {
                listBoxStorages.Items.Add(_storage.Keys[i]);
            }

            if (listBoxStorages.Items.Count > 0 && (index == -1 || index >= listBoxStorages.Items.Count))
            {
                listBoxStorages.SelectedIndex = 0;
            }

            else if (listBoxStorages.Items.Count > 0 && index > -1 && index < listBoxStorages.Items.Count)
            {
                listBoxStorages.SelectedIndex = index;
            }
        }
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxSet.Text))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _storage.AddSet(textBoxSet.Text);
            ReloadObjects();
        }
        private void listBoxObjects_SelectedIndexChanged(object sender, EventArgs e)
        {
            pictureBoxCruiser.Image = _storage[listBoxStorages.SelectedItem?.ToString() ?? string.Empty]?.ShowCruisers();
        }
        private void ButtonDelObject_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            if (MessageBox.Show($"Удалить объект{listBoxStorages.SelectedItem}?", "Удаление", MessageBoxButtons.YesNo,
                MessageBoxIcon.Question) == DialogResult.Yes)
            {
                _storage.DelSet(listBoxStorages.SelectedItem.ToString()
                ?? string.Empty);
                ReloadObjects();
            }
        }
        private void ButtonAddCruiser_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ?? string.Empty];
            if (obj == null)
            {
                return;
            }
            CruiserForm form = new();
            if (form.ShowDialog() == DialogResult.OK)
            {
                if (obj + form.SelectedCruiser)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBoxCruiser.Image = obj.ShowCruisers();
                }
                else
                {
                    MessageBox.Show("Не удалось добавить объект");
                }
            }
        }
        private void ButtonRemoveCruiser_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            if (MessageBox.Show("Удалить объект?", "Удаление",
            MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.No)
            {
                return;
            }
            int pos = Convert.ToInt32(textBoxCruiser.Text);
            if (obj - pos != null)
            {
                MessageBox.Show("Объект удален");
                pictureBoxCruiser.Image = obj.ShowCruisers();
            }
            else
            {
                MessageBox.Show("Не удалось удалить объект");
            }
        }
        private void ButtonRefreshCollection_Click(object sender, EventArgs e)
        {
            if (listBoxStorages.SelectedIndex == -1)
            {
                return;
            }
            var obj = _storage[listBoxStorages.SelectedItem.ToString() ??
            string.Empty];
            if (obj == null)
            {
                return;
            }
            pictureBoxCruiser.Image = obj.ShowCruisers();
        }
    }
}